# -*- coding: utf-8 -*-
import factory
from factory import fuzzy
from allink_core.core.test.factories import FilerImageFactory
from ..models import AllinkLegacyLink


class AllinkLegacyLinkFactory(factory.DjangoModelFactory):

    class Meta:
        model = AllinkLegacyLink

    old = '/old-path/'
    overwrite = '/new-path/'
    active = True
    match_subpages = False
    language = 'de'
    # last_test_result =
    # last_test_date =
    # skip_redirect_when_logged_in = False


# Generated by Django 2.2.13 on 2020-10-27 18:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('news', '0006_auto_20201001_1556'),
    ]

    operations = [
        migrations.AddField(
            model_name='newstranslation',
            name='teaser_link_url',
            field=models.URLField(blank=True, default='', help_text='Provide a valid URL to an external website.', max_length=500, verbose_name='External link'),
        ),
    ]



# Generated by Django 2.2.17 on 2021-01-14 08:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('locations', '0010_auto_20201130_1447'),
    ]

    operations = [
        migrations.RenameField(
            model_name='locations',
            old_name='country',
            new_name='country_old',
        ),
    ]



# -*- coding: utf-8 -*-
from cms.toolbar_pool import toolbar_pool
from cms.toolbar_base import CMSToolbar

from allink_core.core.cms_toolbars import AllinkBaseModifierMixin
from allink_core.core.loading import get_model

Partner = get_model('partner', 'Partner')


class PartnerToolbar(AllinkBaseModifierMixin, CMSToolbar):
    model = Partner
    app_label = Partner._meta.app_label


toolbar_pool.register(PartnerToolbar)



# Generated by Django 2.2.13 on 2020-11-12 08:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('allink_gallery', '0002_auto_20201102_1644'),
    ]

    operations = [
        migrations.AddField(
            model_name='allinkgalleryimageplugin',
            name='width_alias',
            field=models.CharField(blank=True, max_length=50, null=True, verbose_name='Width Alias'),
        ),
        migrations.AddField(
            model_name='allinkgalleryplugin',
            name='width_alias',
            field=models.CharField(blank=True, max_length=50, null=True, verbose_name='Width Alias'),
        ),
    ]



# Generated by Django 2.2.17 on 2021-01-14 09:02

from django.db import migrations
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist


def forwards_func(apps, schema_editor):
    BaseLocations = apps.get_model('locations', 'Locations')
    LocationsTranslation = apps.get_model('locations', 'LocationsTranslation')

    for object in BaseLocations.objects.all():
        object_translation = LocationsTranslation.objects.get(
            master_id=object.pk,
            language_code=settings.LANGUAGE_CODE
        )
        if object_translation is not None:
            object_translation.country = object.country_old
            object_translation.save()


def backwards_func(apps, schema_editor):
    BaseLocations = apps.get_model('locations', 'Locations')
    LocationsTranslation = apps.get_model('locations', 'LocationsTranslation')

    for object in BaseLocations.objects.all():
        translation = _get_translation(object, LocationsTranslation)
        object.country_old = translation.country
        object.save()   # Note this only calls Model.save()


def _get_translation(object, MyModelTranslation):
    translations = MyModelTranslation.objects.filter(master_id=object.pk)
    try:
        # Try default translation
        return translations.get(language_code=settings.LANGUAGE_CODE)
    except ObjectDoesNotExist:
        try:
            # Try default language
            return translations.get(language_code=settings.PARLER_DEFAULT_LANGUAGE_CODE)
        except ObjectDoesNotExist:
            # Maybe the object was translated only in a specific language?
            # Hope there is a single
            return translations.get()


class Migration(migrations.Migration):

    dependencies = [
        ('locations', '0012_locationstranslation_country'),
    ]

    operations = [
        migrations.RunPython(forwards_func, backwards_func),
        migrations.RemoveField(
            model_name='locations',
            name='country_old',
        ),
    ]



# -*- coding: utf-8 -*allink_core/core/models/base_plugins.py-
from allink_core.apps.partner.abstract_models import (
    BasePartner,
    BasePartnerTranslation,
    BasePartnerAppContentPlugin,
)
from allink_core.core.loading import get_model
from allink_core.core.loading import is_model_registered

__all__ = []

if not is_model_registered('partner', 'Partner'):
    class Partner(BasePartner):
        pass

    __all__.append('Partner')

if not is_model_registered('partner', 'PartnerTranslation'):
    class PartnerTranslation(BasePartnerTranslation):
        pass

    __all__.append('PartnerTranslation')

if not is_model_registered('partner', 'PartnerAppContentPlugin'):
    class PartnerAppContentPlugin(BasePartnerAppContentPlugin):
        data_model = get_model('partner', 'Partner')

    __all__.append('PartnerAppContentPlugin')



# Generated by Django 2.2.16 on 2021-03-16 22:13

from django.db import migrations, models
import django.db.models.deletion
import djangocms_text_ckeditor.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0022_auto_20180620_1551'),
    ]

    operations = [
        migrations.CreateModel(
            name='AllinkListMulticolPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='allink_list_multicol_allinklistmulticolplugin', serialize=False, to='cms.CMSPlugin')),
                ('text', djangocms_text_ckeditor.fields.HTMLField(blank=True, help_text='Numbered and bulleted lists in this text will automatically be displayed in multiple columns.', null=True, verbose_name='Text')),
                ('column_mode', models.IntegerField(choices=[(0, 'Two Columns'), (1, 'Three Columns')], default=0, verbose_name='Columns')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



# Generated by Django 2.2.16 on 2021-02-10 13:26

from django.db import migrations
import django.db.models.deletion
import filer.fields.file


class Migration(migrations.Migration):

    dependencies = [
        ('filer', '0012_file_mime_type'),
        ('allink_categories', '0002_auto_20191002_1951'),
    ]

    operations = [
        migrations.AddField(
            model_name='allinkcategory',
            name='logo',
            field=filer.fields.file.FilerFileField(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='allink_categories_allinkcategory_logo', to='filer.File', verbose_name='Logo'),
        ),
    ]



from django.http import HttpResponsePermanentRedirect
from .utils import legacy_redirect


class AllinkLegacyRedirectMiddleware(object):
    """
    If the request matches one of the urls configured in AllinkLegacyLink a HttpResponsePermanentRedirect returned to
    the appropriate url.

    Google Click ids will be preserved.
    """

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)
        return legacy_redirect(request, response)





# Generated by Django 2.2.13 on 2020-10-01 15:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('people', '0004_auto_20191002_2020'),
    ]

    operations = [
        migrations.AddField(
            model_name='peopleappcontentplugin',
            name='load_more_link',
            field=models.URLField(blank=True, help_text='Link for Button Below Items if custom URL is chosen', null=True, verbose_name='Custom Load More Link'),
        ),
        migrations.AlterField(
            model_name='peopleappcontentplugin',
            name='pagination_type',
            field=models.CharField(choices=[('no', 'None'), ('load', 'Add "Load more"-Button'), ('load_rest', 'Add "Load all"-Button'), ('load_url', 'Add "Custom URL"-Button')], default=('no', 'None'), max_length=50, verbose_name='Pagination Type'),
        ),
    ]



# Generated by Django 2.2.13 on 2020-11-02 16:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('allink_gallery', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='allinkgalleryplugin',
            name='auto_start_enabled',
            field=models.BooleanField(default=False, help_text='This option enables autoplay for this gallery.', verbose_name='Autostart'),
        ),
    ]



from unittest import mock
from django.test.testcases import TestCase, override_settings
from django.template import Context, Template

from easy_thumbnails.exceptions import InvalidImageFormatError
from allink_core.core.test.base import GenericPluginMixin
from allink_core.core.utils import get_ratio_w_h, get_height_from_ratio
from allink_core.core.test.factories import FilerImageFactory
from allink_core.core.templatetags.allink_image_tags import (
    get_sizes_from_width_alias, get_width_alias_from_column_plugin, render_image,
)
from allink_core.core_apps.allink_image.cms_plugins import CMSAllinkImagePlugin


class MockThumbnailer(mock.Mock):
    def get_thumbnail(*args, **kwargs):
        return None


""" as reference
THUMBNAIL_WIDTH_ALIASES = {
    '1-of-1': {
        'xs': {'width': 450, 'ratio': '3-2'},
        'sm': {'width': 1200, 'ratio': '3-2'},
        'xl': {'width': 1500, 'ratio': '3-2'}
    },
    '1-of-2': {
        'xs': {'width': 450, 'ratio': '3-2'},
        'sm': {'width': 650, 'ratio': '3-2'},
        'xl': {'width': 900, 'ratio': '3-2'}
    },
    '2-of-3': {
        'xs': {'width': 450, 'ratio': '3-2'},
        'sm': {'width': 600, 'ratio': '3-2'},
        'xl': {'width': 800, 'ratio': '3-2'}
    },
    '1-of-3': {
        'xs': {'width': 450, 'ratio': '3-2'},
        'sm': {'width': 400, 'ratio': '3-2'},
        'xl': {'width': 400, 'ratio': '3-2'}
    },
    '1-of-4': {
        'xs': {'width': 450, 'ratio': '3-2'},
        'sm': {'width': 500, 'ratio': '3-2'},
        'xl': {'width': 500, 'ratio': '3-2'}
    },
    'test-original': {
        'xs': {'width': 450, 'ratio': 'x-y'},
        'sm': {'width': 500, 'ratio': 'x-y'},
        'xl': {'width': 500, 'ratio': 'x-y'}
    },
}
"""

THUMBNAIL_WIDTH_ALIASES_MISSING = {
    '1-of-1': {
        'xs': {'width': 450, 'ratio': '3-2'},
        'sm': {'width': 1200, 'ratio': '3-2'},
        'xl': {'width': 1500, 'ratio': '3-2'}
    },
    '1-of-2': {
        'xs': {'width': 450, 'ratio': '3-2'},
        # 'sm': {'width': 650, 'ratio': '3-2'},  missing
        'xl': {'width': 900, 'ratio': '3-2'}
    },
}


class ImageTagsUtilsTestCase(TestCase):

    def test_get_ratio_w_h(self):
        ratio = '3-2'
        expected_w_h = (3, 2)
        result = get_ratio_w_h(ratio)
        self.assertEqual(expected_w_h, result)

    def test_get_height_from_ratio(self):
        width = 450
        ratio_w = 3
        ratio_h = 2

        expected_h = width * ratio_h / ratio_w  # 300

        result = get_height_from_ratio(width, ratio_w, ratio_h)
        self.assertEqual(expected_h, result)

    def test_get_sizes_from_width_alias(self):
        alias = '1-of-1'
        expected_sizes = [
            ('xs', (450, 300)),
            ('sm', (1200, 800)),
            ('xl', (1500, 1000)),
        ]
        result = get_sizes_from_width_alias(alias, image=None)
        self.assertListEqual(expected_sizes, result)

    def test_get_sizes_from_width_alias_original(self):
        alias = 'test-original'
        expected_sizes = [
            ('xs', (450, 225)),
            ('sm', (500, 250)),
            ('xl', (500, 250)),
        ]

        image = mock.MagicMock()
        image.width = 1000
        image.height = 500

        result = get_sizes_from_width_alias(alias, image=image)
        self.assertListEqual(expected_sizes, result)


class ImageTagsImagePluginUtilsTestCase(GenericPluginMixin, TestCase):
    plugin_class = CMSAllinkImagePlugin

    def test_get_width_alias_from_column_plugin_no_context_3(self):
        result = get_width_alias_from_column_plugin(self.plugin_model_instance)
        self.assertEqual('2-of-3', result)


class ImageTagsImagePluginContextTestCase(TestCase):
    """
    used width_alias:

    '1-of-4': {
        'xs': {'width': 450, 'ratio': '3-2'},
        'sm': {'width': 500, 'ratio': '3-2'},
        'xl': {'width': 500, 'ratio': '3-2'}
    },

    expected context:
    {
        'instance': {'id': 1},

        'image', Image:

        'icon_enabled': True,
        'bg_enabled': True,
        'bg_color': None,
        'lazyload_enabled': True,
        'alt_text': '',
        'vh_enabled': False,

        'ratio_percent_xs': '66.66666666666666%',
        'ratio_vh_xs': '66.66666666666666vh',
        'thumbnail_xs', ThumbnailFile: filer_public_thumbnails/filer_public/a6/34/a634ad24-214f-400c-8957-90c3a23d3da0/so.png__450x300.0_q60_HIGH_RESOLUTION_crop-smart_subsampling-2_upscale.jpg>,

        'ratio_percent_sm': '66.66666666666666%',
        'ratio_vh_sm': '66.66666666666666vh',
        'thumbnail_sm', ThumbnailFile: filer_public_thumbnails/filer_public/a6/34/a634ad24-214f-400c-8957-90c3a23d3da0/so.png__600x400.0_q60_HIGH_RESOLUTION_crop-smart_subsampling-2_upscale.jpg>,

        'ratio_percent_xl': '66.66666666666667%',
        'ratio_vh_xl': '66.66666666666667vh',
        'thumbnail_xl', ThumbnailFile: filer_public_thumbnails/filer_public/a6/34/a634ad24-214f-400c-8957-90c3a23d3da0/so.png__800x533.3333333333334_q60_HIGH_RESOLUTION_crop-smart_subsampling-2_upscale.jpg>
    }
    """

    width_alias = '1-of-4'

    def setUp(self):
        super().setUp()
        self.dummy_plugin = mock.Mock(id=1)

        self.context = {
            'instance': self.dummy_plugin
        }
        self.image = FilerImageFactory()

    def test_context_default_values(self):
        context = render_image(self.context, self.image, width_alias=self.width_alias)

        self.assertEqual(context.get('instance'), self.dummy_plugin)
        self.assertEqual(context.get('icon_enabled'), False)
        self.assertEqual(context.get('bg_enabled'), True)
        self.assertEqual(context.get('bg_color'), None)
        self.assertEqual(context.get('lazyload_enabled'), True)
        self.assertEqual(context.get('vh_enabled'), False)

    def test_context_ratio_values_xs(self):
        # 'xs': {'width': 450, 'ratio': '3-2'},
        w = 450
        h = w / 3 * 2

        expected_ration = '{}'.format(h / w * 100)  # 66.66666666666666
        context = render_image(self.context, self.image, width_alias=self.width_alias)

        self.assertEqual(context.get('ratio_percent_sm'), '{}%'.format(expected_ration))
        self.assertEqual(context.get('ratio_vh_sm'), '{}vh'.format(expected_ration))

    def test_context_ratio_values_sm(self):
        # 'sm': {'width': 500, 'ratio': '3-2'},
        w = 500
        h = w / 3 * 2

        expected_ration = '{}'.format(h / w * 100)  # 66.66666666666666
        context = render_image(self.context, self.image, width_alias=self.width_alias)

        self.assertEqual(context.get('ratio_percent_xs'), '{}%'.format(expected_ration))
        self.assertEqual(context.get('ratio_vh_xs'), '{}vh'.format(expected_ration))

    def test_context_ratio_values_xl(self):
        # 'xl': {'width': 500, 'ratio': '3-2'}
        w = 500
        h = w / 3 * 2

        expected_ration = '{}'.format(h / w * 100)  # 66.66666666666667
        context = render_image(self.context, self.image, width_alias=self.width_alias)

        self.assertEqual(context.get('ratio_percent_xl'), '{}%'.format(expected_ration))
        self.assertEqual(context.get('ratio_vh_xl'), '{}vh'.format(expected_ration))

    @override_settings(THUMBNAIL_WIDTH_ALIASES=THUMBNAIL_WIDTH_ALIASES_MISSING)
    def test_missing_width_alias_in_second_render(self):
        """
        if two images are rendered on the same page with different width aliases
        and the second one is missing one size, the context of the first one is used. Which should not be used.
        """
        # first render
        context = render_image(self.context, self.image, width_alias='1-of-1')
        # use context from first render (ike this is what is the case when calling render_image in a template twice)
        context = render_image(context, self.image, width_alias='1-of-2')
        self.assertIsNone(context.get('thumbnail_sm'))

    @mock.patch('allink_core.core.templatetags.allink_image_tags.get_unique_key')
    def test_picture_id_instance(self, mock_get_unique_key):
        mock_get_unique_key.return_value = 1
        context = render_image(self.context, self.image, width_alias='1-of-1')
        self.assertEqual(context.get('picture_id'), f'picture-{self.image.id}-1-1500-1000')

    @mock.patch('allink_core.core.templatetags.allink_image_tags.get_unique_key')
    def test_picture_id_instance_none(self, mock_get_unique_key):
        mock_get_unique_key.return_value = 1
        ctx_no_instance = {}
        context = render_image(ctx_no_instance, self.image, width_alias='1-of-1')

        self.assertEqual(context.get('picture_id'), f'picture-{self.image.id}-1-1500-1000')

    @mock.patch('allink_core.core.templatetags.allink_image_tags.get_thumbnailer', MockThumbnailer)
    def test_image_does_not_exist(self):
        """ if the image does not exist, no exception should not thrown """
        with mock.patch.object(MockThumbnailer, 'get_thumbnail') as mock_method:
            mock_method.side_effect = InvalidImageFormatError
            context = render_image(self.context, self.image, width_alias=self.width_alias)
            self.assertIsNone(context.get('thumbnail_xl'))

    @mock.patch('allink_core.core.templatetags.allink_image_tags.get_thumbnailer', MockThumbnailer)
    def test_thumbnail_options_default_values(self):
        # 'xl': {'width': 500, 'ratio': '3-2'}
        w = 500
        h = w / 3 * 2

        expected_thumbnail_options = {
            'crop': 'smart', 'bw': False, 'upscale': True, 'HIGH_RESOLUTION': True, 'zoom': None,
            'subject_location': self.image.subject_location, 'size': (w, h),
        }
        with mock.patch.object(MockThumbnailer, 'get_thumbnail') as mock_method:
            render_image(self.context, self.image, width_alias=self.width_alias)

            mock_method.assert_called_with(expected_thumbnail_options)
            self.assertEqual(mock_method.call_count, 3)


class ImageTagsImagePluginRenderTestCase(TestCase):
    """
    used width_alias:

    '1-of-4': {
        'xs': {'width': 450, 'ratio': '3-2'},
        'sm': {'width': 500, 'ratio': '3-2'},
        'xl': {'width': 500, 'ratio': '3-2'}
    },
    """

    def setUp(self):
        super().setUp()
        self.dummy_plugin = mock.Mock(id=1)
        self.context = {
            'instance': self.dummy_plugin
        }
        self.image = FilerImageFactory()
        self.sizes = {
            'xs': "450x{}".format(450 / 3 * 2),
            'sm': "500x{}".format(500 / 3 * 2),
            'xl': "500x{}".format(500 / 3 * 2),
        }

    def test_context_render_all_image_urls(self):
        context = Context({
            'instance': self.dummy_plugin,
            'image': self.image,

        })
        template_to_render = Template(
            '{% load allink_image_tags %}'
            '{% render_image image width_alias="1-of-4" %}'
        )
        rendered_template = template_to_render.render(context)

        for size in self.sizes.items():
            self.assertIn('{}__{}'.format(self.image.original_filename, size[1]), rendered_template)



from collections import OrderedDict
from urllib.parse import urlparse, urlunparse
from django.db.models import Q
from django.http import HttpResponsePermanentRedirect


def sort_get_params(path):
    """
    sorts GET parameters alphabetically in a url
    """
    url = urlparse(path)
    if url.query:
        get_parameters = OrderedDict(gp.split("=") for gp in url.query.split("&"))
        sorted_get_parameters = dict(sorted(get_parameters.items(), key=lambda x: x[0].lower()))
        sorted_query_string = '&'.join(['%s=%s' % (key, value) for (key, value) in sorted_get_parameters.items()])
        url = url._replace(query=sorted_query_string)
    return urlunparse(url)


def pre_and_append_slash(path):
    url = urlparse(path)
    if not url.path.startswith('/'):
        url = url._replace(path='/' + url.path)
    if not url.path.endswith('/'):
        url = url._replace(path=url.path + '/')
    return urlunparse(url)


def append_gclid(path, get_params):
    """
    preserve Google Click Identifier in path
    """
    url = urlparse(path)

    if 'gclid' in get_params:
        base = url.query + '&' if url.query else url.query
        url = url._replace(query='{}gclid={}'.format(base, get_params['gclid']))
    return urlunparse(url)


def fetch_legacy_redirect_match_subpages_on_request_path(request):
    """
    check for links which match subpages
    this must be done after the check on an exact match (including) GET parameter, because subpages are less specific.
    :returns the first item in the queryset or None
    """
    from .models import AllinkLegacyLink

    # build up array with path parts
    # e.g. ['en','agency','contact']
    path_bits = request.path.split('/')[1:-1]
    length = len(path_bits)
    olds = []

    # omit final bit because we already
    # tested for it in our first try
    # e.g. ['/en/','/en/agency/']
    for i in range(1, length):
        olds.append('/%s/' % ('/').join(path_bits[:i]))

    # if we got multiple links with match_subpages
    # enabled (e.g. /en/ and /en/agency/), return
    # the longer one
    legacy_link = AllinkLegacyLink.objects.filter(
        old__in=olds,
        match_subpages=True,
        active=True
    ).order_by('old').last()

    return legacy_link


def fetch_legacy_redirect_on_request_path(request):
    """
    fetches LegacyLink from db if old path matches the request path
    :returns the first item in the queryset or None
    """
    from .models import AllinkLegacyLink

    legacy_link = AllinkLegacyLink.objects.filter(Q(old=request.get_full_path()), active=True).first()

    return legacy_link


def legacy_redirect(request, response):
    """
    Checks for a request if a corresponding entry in LegacyRedirect exist and
    :returns the appropriate HttpResponsePermanentRedirect response or the original response
    """
    legacy_link = fetch_legacy_redirect_on_request_path(request=request)

    if not legacy_link:
        legacy_link = fetch_legacy_redirect_match_subpages_on_request_path(request)

    if not legacy_link:
        return response

    # if user is logged in, skip redirect
    if legacy_link.skip_redirect_when_logged_in and request.user.is_authenticated:
        return response

    new_path = legacy_link.link
    new_path = append_gclid(new_path, request.GET)

    return HttpResponsePermanentRedirect(new_path)



from cms import api
from allink_core.core.test.base import PageApphookMixin
from django.test.testcases import TestCase
from allink_core.core_apps.allink_image.cms_plugins import CMSAllinkImagePlugin
from allink_core.core_apps.allink_content.cms_plugins import CMSAllinkContentPlugin, CMSAllinkContentColumnPlugin
from allink_core.core.templatetags.allink_image_tags import (
    get_width_alias_from_column_plugin,
)

class ImageTagsContentPluginUtilsTestCase(PageApphookMixin, TestCase):
    template = 'col-1'
    position = 1  #

    def setUp(self):
        super().setUp()
        # AllinkContentPlugin, template='col-1'
        self.content_plugin = api.add_plugin(
            self.placeholder,
            CMSAllinkContentPlugin,
            self.language,
            template=self.template
        )
        # AllinkContentColumnPlugin, template='col-1'
        column_amount = CMSAllinkContentPlugin.model.get_template_column_count(self.content_plugin.template)

        for x in range(int(column_amount)):
            api.add_plugin(
                self.placeholder,
                CMSAllinkContentColumnPlugin,
                self.language,
                target=self.content_plugin
            )

        self.column_plugin = self.content_plugin.get_children()[self.position - 1]

        # AllinkImagePlugin, inside column plugin
        self.image_plugin = api.add_plugin(
            self.placeholder,
            CMSAllinkImagePlugin,
            self.language,
            target=self.column_plugin
        )

    def test_get_width_alias_from_column_plugin_no_context_1(self):
        """ template col-1 """

        result = get_width_alias_from_column_plugin(self.image_plugin)
        self.assertEqual('1-of-1', result)

    def test_get_width_alias_from_column_plugin_no_context_1_1(self):
        self.content_plugin.template = 'col-1-1'
        self.content_plugin.save()

        result = get_width_alias_from_column_plugin(self.image_plugin)
        self.assertEqual('1-of-2', result)

    def test_get_width_alias_from_column_plugin_no_context_1_2_pos_0(self):
        self.content_plugin.template = 'col-1-2'
        self.content_plugin.save()

        result = get_width_alias_from_column_plugin(self.image_plugin)
        self.assertEqual('1-of-3', result)

    def test_get_width_alias_from_column_plugin_no_context_1_2_pos_1(self):
        self.content_plugin.template = 'col-1-2'
        self.content_plugin.save()
        self.column_plugin.position = 1
        self.column_plugin.save()

        result = get_width_alias_from_column_plugin(self.image_plugin)
        self.assertEqual('2-of-3', result)

    def test_get_width_alias_from_column_plugin_no_context_2_1_pos_0(self):
        self.content_plugin.template = 'col-2-1'
        self.content_plugin.save()

        result = get_width_alias_from_column_plugin(self.image_plugin)
        self.assertEqual('2-of-3', result)

    def test_get_width_alias_from_column_plugin_no_context_2_1_pos_1(self):
        self.content_plugin.template = 'col-2-1'
        self.content_plugin.save()
        self.column_plugin.position = 1
        self.column_plugin.save()

        result = get_width_alias_from_column_plugin(self.image_plugin)
        self.assertEqual('1-of-3', result)

    def test_get_width_alias_from_column_plugin_no_context_3(self):
        self.content_plugin.template = 'col-3'
        self.content_plugin.save()

        result = get_width_alias_from_column_plugin(self.image_plugin)
        self.assertEqual('1-of-3', result)

    def test_get_width_alias_from_column_plugin_no_context_4(self):
        self.content_plugin.template = 'col-4'
        self.content_plugin.save()

        result = get_width_alias_from_column_plugin(self.image_plugin)
        self.assertEqual('1-of-4', result)



# -*- coding: utf-8 -*-
from allink_core.core.models.managers import AllinkCategoryModelQuerySet


class PartnerQuerySet(AllinkCategoryModelQuerySet):
    pass


PartnerManager = PartnerQuerySet.as_manager



from cms.models import CMSPlugin
from django.db import models
from djangocms_text_ckeditor.fields import HTMLField


class AllinkListMulticolPlugin(CMSPlugin):
    COLUMN_CHOICES = [
        (0, 'Two Columns'),
        (1, 'Three Columns'),
    ]

    text = HTMLField(
        'Text',
        help_text='Numbered and bulleted lists in this text will automatically be displayed in multiple columns.',
        blank=True,
        null=True,
    )

    column_mode = models.IntegerField(
        verbose_name='Columns',
        choices=COLUMN_CHOICES,
        default=0,
    )



# Generated by Django 2.2.16 on 2020-11-13 08:09

import cms.models.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0022_auto_20180620_1551'),
        ('people', '0005_auto_20201001_1556'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='peopleappcontentplugin',
            name='load_more_link',
        ),
        migrations.AddField(
            model_name='peopleappcontentplugin',
            name='load_more_internallink',
            field=cms.models.fields.PageField(blank=True, help_text='Link for Button Below Items if custom URL is chosen', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='load_more_internallink_people', to='cms.Page', verbose_name='Custom Load More Link'),
        ),
        migrations.AddField(
            model_name='peopletranslation',
            name='teaser_link_url',
            field=models.URLField(blank=True, help_text='Provide a valid URL to an external website.', max_length=500, null=True, verbose_name='External link'),
        ),
    ]



from django.test.testcases import TestCase
from ..utils import camelcase_to_separated_lowercase

class UtilsTestCase(TestCase):
    def test_camelcase_to_separated_lowercase(self):
        self.assertEqual(camelcase_to_separated_lowercase('SomeCameCasedString', '-'), 'some-came-cased-string')



# Generated by Django 2.2.19 on 2021-04-22 18:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('allink_legacy_redirect', '0003_auto_20191016_0804'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='allinklegacylink',
            name='redirect_when_logged_out',
        ),
        migrations.AddField(
            model_name='allinklegacylink',
            name='skip_redirect_when_logged_in',
            field=models.BooleanField(default=False, help_text='If True, current site will not redirect when user is logged in. If False, the page will be redirected.', verbose_name='Skip redirect when logged in'),
        ),
    ]



from cms.models import CMSPlugin
from django.db import models

from django.core.exceptions import ValidationError
from django.utils.safestring import mark_safe
import re

from filer.fields.file import FilerFileField

from allink_core.core.models import AllinkLinkFieldsModel


class AllinkImageSVGPlugin(AllinkLinkFieldsModel, CMSPlugin):
    picture = FilerFileField(
        verbose_name='Image',
        blank=True,
        null=True,
        on_delete=models.PROTECT,
        related_name='%(app_label)s_%(class)s_image',
    )

    is_inline = models.BooleanField(
        'Is inline',
        default=False,
        help_text="Check if the SVG will be inlined.",
    )

    is_fullwidth = models.BooleanField(
        'Is Fullwidth',
        default=True,
        help_text="Stretches the SVG to 100% width.",
    )

    def render_svg(self, **kwargs):
        """
        reads svg image and extracts <svg> tag.
        if not found or can not be read en empty string will be returned
        """
        if not self.picture:
            return ''

        pattern = r'<svg(.*)svg>'
        try:
            file_string = self.picture.file.read().decode("utf-8")
            if file_string:
                return mark_safe(re.search(pattern, file_string, flags=re.DOTALL)[0])
            else:
                return ''
        except OSError:
            return ''

    def clean(self):
        if self.picture and self.picture.extension not in ['svg']:
            raise ValidationError(
                'Incorrect file type: %(value)s',
                params={'value': self.picture.extension},
            )



from cms import api
from allink_core.core.test.base import PageApphookMixin
from django.test.testcases import TestCase
from allink_core.core.templatetags.allink_image_tags import get_width_alias_from_section_plugin
from allink_core.core_apps.allink_teaser.cms_plugins import CMSAllinkTeaserGridContainerPlugin, CMSAllinkTeaserPlugin


class ImageTagsSectionPluginUtilsTestCase(PageApphookMixin, TestCase):
    """
    As AllinkBaseSectionPlugin is an abstract model, we test the behavior regarding render_image
    on a CMSAllinkTeaserGridContainerPlugin.
    """
    colums = '1-of-3'
    position = 1  #

    def setUp(self):
        super().setUp()
        # CMSAllinkTeaserGridContainerPlugin, column='1-of-3'
        self.content_plugin = api.add_plugin(
            self.placeholder,
            CMSAllinkTeaserGridContainerPlugin,
            self.language,
            columns=self.colums
        )

        api.add_plugin(
            self.placeholder,
            CMSAllinkTeaserPlugin,
            self.language,
            target=self.content_plugin
        )

        self.teaser_plugin = self.content_plugin.get_children()[self.position - 1]

    def test_get_width_alias_from_section_plugin(self):
        result = get_width_alias_from_section_plugin(self.teaser_plugin)
        self.assertEqual('1-of-3', result)



# Generated by Django 2.2.13 on 2020-11-03 13:55

import cms.models.fields
from django.db import migrations
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0022_auto_20180620_1551'),
        ('locations', '0008_auto_20201102_1908'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='locationsappcontentplugin',
            name='load_more_link',
        ),
        migrations.AddField(
            model_name='locationsappcontentplugin',
            name='load_more_internallink',
            field=cms.models.fields.PageField(blank=True, help_text='Link for Button Below Items if custom URL is chosen', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='load_more_internallink_locations', to='cms.Page', verbose_name='Custom Load More Link'),
        ),
    ]



# -*- coding: utf-8 -*-
from allink_core.core.views import AllinkBasePluginLoadMoreView, AllinkBaseDetailView
from allink_core.core.loading import get_model

Partner = get_model('partner', 'Partner')
PartnerAppContentPlugin = get_model('partner', 'PartnerAppContentPlugin')


class PartnerPluginLoadMore(AllinkBasePluginLoadMoreView):
    model = Partner
    plugin_model = PartnerAppContentPlugin


class PartnerDetail(AllinkBaseDetailView):
    model = Partner



# Generated by Django 2.2.13 on 2020-11-02 19:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('locations', '0007_locationstranslation_teaser_link_url'),
    ]

    operations = [
        migrations.AlterField(
            model_name='locationstranslation',
            name='teaser_link_url',
            field=models.URLField(blank=True, help_text='Provide a valid URL to an external website.', max_length=500, null=True, verbose_name='External link'),
        ),
    ]



# Generated by Django 2.2.17 on 2021-01-14 09:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('locations', '0011_auto_20210114_0855'),
    ]

    operations = [
        migrations.AddField(
            model_name='locationstranslation',
            name='country',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='Country'),
        ),
    ]



# Generated by Django 2.2.16 on 2020-10-30 11:12

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('config', '0004_allinktitleextension_teaser_link_url'),
    ]

    operations = [
        migrations.RunSQL("""DELETE FROM django_migrations WHERE app='contact' AND name='0001_initial';""")
    ]



from django.utils.translation import ugettext_lazy as _
from django import forms
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from .models import AllinkImageSVGPlugin
from allink_core.core.forms.mixins import AllinkInternalLinkFieldMixin
from allink_core.core.forms.fields import SelectLinkField


class AllinkImageSVGPluginForm(AllinkInternalLinkFieldMixin, forms.ModelForm):

    internal_link = SelectLinkField(label='Link Internal', required=False)

    class Meta:
        model = AllinkImageSVGPlugin
        exclude = ('page', 'position', 'placeholder', 'language', 'plugin_type')

    def __init__(self, *args, **kwargs):
        super(AllinkImageSVGPluginForm, self).__init__(*args, **kwargs)


@plugin_pool.register_plugin
class CMSAllinkImageSVGPlugin(CMSPluginBase):
    name = 'SVG Image'
    module = 'Generic'
    render_template = 'allink_image_svg/content.html'
    model = AllinkImageSVGPlugin
    form = AllinkImageSVGPluginForm

    def get_fieldsets(self, request, obj=None):
        fieldsets = [
            (None, {
                'fields': [
                    'picture',
                    ('is_inline', 'is_fullwidth')
                ]
            }),
            ('Link settings', {
                'classes': ('collapse',),
                'fields': (
                    'internal_link',
                    'link_url',
                    ('link_mailto', 'link_phone'),
                    'link_anchor',
                    'link_file',
                    'link_target',
                )
            }),
        ]

        return fieldsets

    def render(self, context, instance, placeholder):
        context['instance'] = instance
        context['placeholder'] = placeholder
        return context



# Generated by Django 2.2.16 on 2021-03-26 10:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('allink_image_svg', '0002_auto_20210325_1703'),
    ]

    operations = [
        migrations.AddField(
            model_name='allinkimagesvgplugin',
            name='is_fullwidth',
            field=models.BooleanField(default=True, help_text='Stretches the SVG to 100% width.', verbose_name='Is Fullwidth'),
        ),
    ]



from django.http import HttpResponse
from django.contrib.auth.models import AnonymousUser
from django.test.client import RequestFactory
from allink_core.core.test.factories import UserFactory
from django.test.testcases import TestCase
from .factories import AllinkLegacyLinkFactory
from ..utils import (
    pre_and_append_slash, sort_get_params, append_gclid, legacy_redirect, fetch_legacy_redirect_on_request_path,
    fetch_legacy_redirect_match_subpages_on_request_path, )


class LegacyRedirectUtils(TestCase):

    def test_pre_and_append_slash(self):
        path = 'some/incomplete-path?hello=3&you=foo'

        self.assertEqual(pre_and_append_slash(path), '/some/incomplete-path/?hello=3&you=foo')

    def test_sort_get_params(self):
        path = 'some/path/?C=3&a=1&b=Hello'
        self.assertEqual(sort_get_params(path), 'some/path/?a=1&b=Hello&C=3')

    def test_append_gclid(self):
        path = 'some/path/?C=3&a=1&b=Hello'
        original_get_params = {'gclid': '123'}
        self.assertEqual(append_gclid(path, original_get_params), 'some/path/?C=3&a=1&b=Hello&gclid=123')

        path = 'some/path/'
        original_get_params = {'gclid': '123'}
        self.assertEqual(append_gclid(path, original_get_params), 'some/path/?gclid=123')

    def test_fetch_legacy_redirect_on_request_path(self):
        expected_ll = AllinkLegacyLinkFactory(old='/some-path/')
        AllinkLegacyLinkFactory(old='/some-other')

        request = RequestFactory()
        self.assertEqual(fetch_legacy_redirect_on_request_path(request.get('/some-path/')), expected_ll)

    def test_fetch_legacy_redirect_on_request_path_get_params(self):
        expected_ll = AllinkLegacyLinkFactory(old='/some/path/?a=1&b=Hello&C=3')
        request = RequestFactory()
        self.assertEqual(fetch_legacy_redirect_on_request_path(request.get('/some/path/?a=1&b=Hello&C=3')),
                         expected_ll)

    def test_fetch_legacy_redirect_on_request_path_does_not_exists(self):
        AllinkLegacyLinkFactory(old='/some-path/')
        request = RequestFactory()
        self.assertIsNone(fetch_legacy_redirect_on_request_path(request.get('/some-other')))

    def test_fetch_legacy_redirect_match_subpages_on_request_path(self):
        AllinkLegacyLinkFactory(old='/some-path/', match_subpages=True)
        expected_ll = AllinkLegacyLinkFactory(old='/some-path/more/specific/', match_subpages=True)

        request = RequestFactory()
        self.assertEqual(
            fetch_legacy_redirect_match_subpages_on_request_path(request.get('/some-path/more/specific/match/')),
            expected_ll)

    def test_legacy_redirect_skip_redirect_when_logged_out_should_redirect(self):
        AllinkLegacyLinkFactory(old='/some-path/', overwrite='/new-path/', skip_redirect_when_logged_in=True)

        request = RequestFactory().get('/some-path/')
        request.user = AnonymousUser()
        response = HttpResponse(content='dummy')

        self.assertEqual(legacy_redirect(request, response).status_code, 301)

    def test_legacy_redirect_skip_redirect_when_logged_in_should_redirect(self):
        AllinkLegacyLinkFactory(old='/some-path/', overwrite='/new-path/', skip_redirect_when_logged_in=True)

        request = RequestFactory().get('/some-path/')
        user = UserFactory()
        request.user = user
        self.client.force_login(user)
        response = HttpResponse(content='dummy')

        self.assertEqual(legacy_redirect(request, response).status_code, 200)
        self.assertEqual(legacy_redirect(request, response).content, b'dummy')



# Generated by Django 2.2.16 on 2021-03-25 17:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('allink_image_svg', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='allinkimagesvgplugin',
            name='is_inline',
            field=models.BooleanField(default=False, help_text='Check if the SVG will be inlined.', verbose_name='Is inline'),
        ),
    ]



# Generated by Django 2.2.16 on 2020-11-30 14:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('people', '0006_auto_20201113_0809'),
    ]

    operations = [
        migrations.AlterField(
            model_name='people',
            name='zip_code',
            field=models.CharField(blank=True, max_length=10, null=True, verbose_name='Zip Code'),
        ),
    ]



# Generated by Django 2.2.16 on 2021-03-25 12:21

import django.contrib.postgres.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0022_auto_20180620_1551'),
        ('allink_teaser', '0008_auto_20201102_1908'),
    ]

    operations = [
        migrations.CreateModel(
            name='AllinkTeaserGridContainerPlugin',
            fields=[
                ('title', models.CharField(blank=True, max_length=255, null=True, verbose_name='Title')),
                ('columns', models.CharField(help_text='Choose columns.', max_length=50, verbose_name='Columns')),
                ('column_order', models.CharField(help_text='Choose a column order.', max_length=50, verbose_name='Column Order')),
                ('anchor', models.CharField(blank=True, help_text='ID of this content section which can be used for anchor reference from links.<br>Note: Only letters, numbers and hyphen. No spaces or special chars.', max_length=255, verbose_name='ID')),
                ('project_css_classes', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(blank=True, max_length=50, null=True), blank=True, null=True, size=None)),
                ('project_css_spacings_top_bottom', models.CharField(blank=True, help_text='Choose a spacing (top and bottom).', max_length=50, null=True, verbose_name='Spacings')),
                ('project_css_spacings_top', models.CharField(blank=True, help_text='Choose a top spacing.', max_length=50, null=True, verbose_name='Spacings top')),
                ('project_css_spacings_bottom', models.CharField(blank=True, help_text='Choose a bottom spacing.', max_length=50, null=True, verbose_name='Spacings bottom')),
                ('cmsplugin_ptr', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='allink_teaser_allinkteasergridcontainerplugin', serialize=False, to='cms.CMSPlugin')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]
